# -*- coding: utf-8 -*-

'''
© 2012-2013 eBay Software Foundation
Authored by: Tim Keefer
Licensed under CDDL 1.0
'''

class ConnectionError(Exception):
    pass

class ConnectionResponseError(Exception):
    pass
